#define ethuploadrev (0.01)

/*

STANDARD #include section goes here
stdio.h
types.h
w7100.h

*/
uint16 linecount_calculated;
uint16 expectedlinecount;
uint8 linelength;
uint8 linedata[16];				// bytes of data as read from one line of the intel.hex file
uint16 lineaddr;				// start address of data from intel.hex line
uint8 linecsum;					// checksum read from line of intel.hex file
uint8 linecsum_calculated; 		// checksum built as data is parsed; checked against linecsum for verification
uint16 filecsum_calculated;     // checksum built over entire file, sent to host to verify reception
uint16 filecsum;
uint16 maxusedaddr;
uint16 minusedaddr;

#define XBYTE ((unsigned char volatile xdata *) 0)
#define SECTOR_SIZE (0x0400)
//#define SECTOR_SIZE (0x0100)   // does not work
#define DEBUG_OFFSET (0x8000)

/****************************************************************************************************************
  Function name: ETH_BurnFirmware() - DEBUG VERSION
  Brief: Copies a binary code image from XDATA to CODE FLASH
  Input: codestart: lowest used address from the parsed .HEX file that was stored as binary in XDATA
         codeend: highest used address from the parsed .hex file
         x_data_offset: location in XDATA that the binary image storage buffer begins
         DEBUG_OFFSET: during debugging the code flash is written from 0x8000 and up to avoid overwriting this routine during the copy (no springboard mode)
  Output: extensive serial debugging information
*****************************************************************************************************************/
void ETH_BurnFirmware( uint16 codestart, uint16 codeend, uint16 x_data_offset )
// x_data_offset is used to indicate the start address in XDATA memory where the firmware.hex image was stored.
// images are stored at 0x8000 typically, but are written to "codestart" in code flash.
{
    uint16 i,addr,vaddr;
	uint8 verify_failed_flag,k;

	printf("ETH-DO-8 Firmware Update (v%1.2f): Copy RAM to CODE FLASH\r\n",ethuploadrev);
	if (codestart%SECTOR_SIZE !=0) printf("codestart (%04X) isn't sector aligned, erasing may be dangerous, proceeding anyway\r\n", codestart);
	if (codeend%SECTOR_SIZE !=0) printf("codeend (%04X) isn't sector aligned, erasing may be dangerous, proceeding anyway\r\n", codeend);
	code_sector_erase(codestart+DEBUG_OFFSET);// line should be: code_sector_erase(codestart);  when debugging done (springboard)

	addr=codestart; vaddr=codestart;
	do
	{
		printf("Sector %04x erasing...",addr+DEBUG_OFFSET);
    	code_sector_erase(addr+DEBUG_OFFSET); // NOTE #3: erases sectors 0+1+2+3
		printf("writing...");
	    do
		{
			code_write_byte(DEBUG_OFFSET+addr, XBYTE[DEBUG_OFFSET+addr]);
			addr++;
			if (addr>codeend) break;
		} while(addr % SECTOR_SIZE != 0);

		printf("verifying...");
		verify_failed_flag = 0;
	    do
		{
			if (verify_failed_flag == 0)
				if (code_read_byte(DEBUG_OFFSET+vaddr) == XBYTE[DEBUG_OFFSET+vaddr])
				{
				}
				else
				{
					verify_failed_flag = 1;
				    printf("FAILED @ %04X\r\n",vaddr+DEBUG_OFFSET);   // NOTE #1: WORKS
				}
			vaddr++;
			if (vaddr>codeend) break;
		} while(vaddr % SECTOR_SIZE != 0);
     	if (verify_failed_flag == 0) printf("Verified sector.\r\n");
		verify_failed_flag=0;
    } while(vaddr<codeend);

	printf("Post-verifying (double check for sector erase size issues)...\r\n");
	addr=codestart; vaddr=codestart;
	do
	{
		verify_failed_flag = 0;
	    do
		{
			if (verify_failed_flag == 0)
				if (code_read_byte(DEBUG_OFFSET+vaddr) == XBYTE[DEBUG_OFFSET+vaddr])
				{
				}
				else
				{
					verify_failed_flag = 1;
				    printf("FAILED @ %04X\r\n",vaddr+DEBUG_OFFSET);  // NOTE #2: Fails on sectors 0,1,2 but works on 3
				}
			vaddr++;
			if (vaddr>codeend) break;
		} while(vaddr % SECTOR_SIZE != 0);
     	if (verify_failed_flag == 0) printf("Verified sector codeending at (%04x).\r\n",vaddr-1);
		verify_failed_flag=0;
    } while(vaddr<codeend);
    printf("ETH-DO-8 Firmware Update: Copy RAM to CODE FLASH (%04X -> %04X) Finished.\r\n", codestart, codeend);
}